
//+------------------------------------------------------------------+
//|                                                 RandomWalker.mq4 |
//+------------------------------------------------------------------+

#property show_inputs

#import "kernel32.dll"
 bool QueryPerformanceCounter(int& counter[]);
#import

extern string ScriptName = "RandomWalker";

int min, popsize = 100;
double x[], y[], fitness[];
//+------------------------------------------------------------------+
int RandomNumber(int start, int step, int stop) {
 int seed;
 int counter[2];
 int values = MathAbs((stop-start)/step);
 if (QueryPerformanceCounter(counter)) {
  seed = MathAbs(counter[0] % (values + 1));
 }
 int res = start+(seed*step);
 return(res);
}
//+------------------------------------------------------------------+
double RandomDouble() { // -5.12,5.12
 
 string str=RandomNumber(-5,1,5) + ".";
 
 if(str!="-5." && str!="5.") {
  str=str+RandomNumber(0,1,9)+RandomNumber(0,1,9);
 }
 else {
  str=str+RandomNumber(0,1,1)+RandomNumber(0,1,1);
 }
 for(int i=0; i<8; i++) {
  str=str+RandomNumber(0,1,9);
 }
 return(StrToDouble(str));
}
//+------------------------------------------------------------------+
double Pi() {
 // found in the mql4 code base
 // copyright "Privalov S.V."
 // http://codebase.mql4.com/4073
 double pi=0.0;
 for(double i=0; i<11; i++) {
  pi+= 1/MathPow(16,i)*(4/(8*i+1)-2/(8*i+4)-1/(8*i+5)-1/(8*i+6));
 }
 return(pi);
}
//+------------------------------------------------------------------+
bool Equals(double double1, double double2) {
 return(NormalizeDouble(double1*1000000,8)==NormalizeDouble(double2*1000000,8));
}
//+------------------------------------------------------------------+
void Initialize() {
 for(int i=0; i<popsize; i++) {
  x[i]=RandomDouble();
  y[i]=RandomDouble();
 }
}
//+------------------------------------------------------------------+
void CalculateFitness() { // http://en.wikipedia.org/wiki/File:Rastrigin_function.png
 for(int i=0; i<popsize; i++) {
  fitness[i]=20+MathPow(x[i],2)+MathPow(y[i],2)-(10*MathCos(2*Pi()*x[i]))-(10*MathCos(2*Pi()*y[i]));
 }
}
//+------------------------------------------------------------------+
void UpdateIndividuals() {
 
 double new[5];
 double bestx = x[min];
 double besty = y[min];
 
 for(int i=0; i<popsize; i++) {
  
  new[0] = bestx+MathPow(10,RandomNumber(-16,1,0));
  new[1] = bestx-MathPow(10,RandomNumber(-16,1,0));
  new[2] = besty+MathPow(10,RandomNumber(-16,1,0));
  new[3] = besty-MathPow(10,RandomNumber(-16,1,0));
  new[4] = RandomDouble();
 
  if(new[0]>5.12)  {new[0]=5.12;}
  if(new[1]<-5.12) {new[1]=-5.12;}
  if(new[2]>5.12)  {new[2]=5.12;}
  if(new[3]<-5.12) {new[3]=-5.12;}
  
  x[i] = new[RandomNumber(0,1,4)];
  y[i] = new[RandomNumber(0,1,4)];
 }
}
//+------------------------------------------------------------------+
//| script program start function                                    |
//+------------------------------------------------------------------+
int start() {

 int StartTime = GetTickCount();
	
 ArrayResize(x,popsize);
 ArrayResize(y,popsize);
 ArrayResize(fitness,popsize);
 
 Initialize();
 CalculateFitness();
	
	int i = 0;
	while(!Equals(fitness[min]*1000000,0)) {
	
	 UpdateIndividuals();
  CalculateFitness();
  min = ArrayMinimum(fitness);
  
	 Comment("\n"+" Current Iteration: "+i+", Fitness: "+fitness[min]+" Best x: "+x[min]+" Best y: "+y[min]);
	 Print("Current Iteration: "+i+", Fitness: "+fitness[min]+" Best x: "+x[min]+" Best y: "+y[min]);
	 i++;
	}
	double ExecutionTime = (GetTickCount() - StartTime) / 1000.0;
	Comment("\n"+" Iterations used: "+i+"\n"+" Execution time: "+DoubleToStr(ExecutionTime,3)+
	"\n"+"\n"+" Best x: "+x[min]+", Best y: "+y[min]+", Fitness: "+fitness[min]);
 return(0);
}
//+------------------------------------------------------------------+

